#!perl -w
#
# Copyright 2003-2004 The Apache Software Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# This script is intended to be called from within the docs build process.
# It gets two parameters, ${input_ext} (e.g. .xml.de) and the full path name
# of the current file.
# It assumes, that the current working directory is the root of the manual.

# The script doesn't use an XML parser (yet?) so care should be taken with
# non-ascii characters.

# TODO:
# - rewrite in java as ant task?

# against fat fingers
use strict;

# for file operations
use FindBin;
use lib $FindBin::Bin;
use Fcntl qw(O_RDONLY O_WRONLY O_CREAT O_TRUNC);
use DocUtil qw(%suffix2lang $ignore_files srcinfo docpath);

# scratch stuff
my (%current, %variants, @pfiles, $curpath);

# check commandline parameters
die "missing parameter(s)\n"
    unless defined $ARGV[0] and defined $ARGV[1];
    #              suffix               srcfile

my ($dirname, $basename, $relative, $metafile) = srcinfo();

# first determine the reality
$curpath = docpath($dirname);
opendir(DIR, $curpath)
    or die "could not open directory '$curpath' ($!), stopped";

@pfiles = sort
       grep /^\Q$basename.\E(?:html\.[^.]+|xml)/
    => grep !/$ignore_files/
    => readdir(DIR);

closedir(DIR)
    or die "could not close directory '$curpath' ($!), stopped";

# now loop over the filelist.
# it contains all files of one language including
# html AND xml variants.
for (@pfiles) {
    my $suffix;
    my $lang = $suffix = /^\Q$basename.\E(?:html|xml)\.([^.]+)/ ? $1 : 'en';
    $lang = $suffix2lang{$suffix} if $suffix2lang{$suffix};

    $variants{$lang} = {};
    $variants{$lang}->{'pdf'} = "$basename.$suffix.pdf"
        if -f docpath("$dirname$basename.$suffix.pdf");

    # the following works, because the list (@pfiles) is sorted, that way
    # html is processed before xml and thus typically we set the
    # parameter first and delete it later.
    if (/^\Q$basename.xml/) {
        delete $variants{$lang}->{'htmlonly'};
    }
    else {
        $variants{$lang}->{'htmlonly'} = 'yes';
    }
}

# create resulting file contents
my $content = <<"XML";
<?xml version="1.0" encoding="UTF-8" ?>

<metafile>
  <basename>$basename</basename>
  <path>$dirname</path>
  <relpath>$relative</relpath>

  <variants>
    @{[
        join "\n    "
        => map
            qq{<variant@{[do{ my $lang = $_;
                map qq( $_="$variants{$lang}->{$_}")
                => keys %{$variants{$lang}};
            }]}>$_</variant>}
        => sort keys %variants
    ]}
  </variants>
</metafile>
XML

# read current variant file if exists
my $existing = '';
$curpath = docpath($metafile);
if (-f $curpath) {
    local $/; # slurp mode /

    sysopen(FILE, $curpath, O_RDONLY)
        or die "could not open file '$curpath' ($!), stopped";

    $existing = <FILE>;

    close(FILE)
        or die "could not close file '$curpath' ($!), stopped";
}

# compare and possibly write new result to disk
unless ($content eq $existing) {
    my $ppath = $metafile;
    $ppath =~ s,^/,,;

    sysopen(FILE, $curpath, O_WRONLY | O_CREAT | O_TRUNC)
        or die "could not open file '$curpath' ($!), stopped";

    print FILE $content
        or die "could not write into file '$curpath' ($!), stopped";

    close(FILE)
        or die "could not close file '$curpath' ($!), stopped";

    # report success
    print "'$ppath' written.\n";
}

__END__
sample file:

<?xml version="1.0" encoding="UTF-8" ?>

<metafile>
  <basename>quickreference</basename>
  <path>/mod/</path>
  <relpath>..</relpath>

  <variants>
    <variant pdf="quickreference.en.pdf">en</variant>
    <variant>ja</variant>
  </variants>
</metafile>
